# Generated by roxygen2: do not edit by hand

export(LVGP_fit)
export(LVGP_plot)
export(LVGP_predict)
export(corr_mat)
export(neg_log_l)
export(to_latent)
import(graphics)
import(lhs)
import(parallel)
import(randtoolbox)
